local ADDON, namespace = ...
local L = namespace.L
local tooltip

local arg = {}

local QUESTION_MARK_ICON = [[Interface\Icons\INV_MISC_QUESTIONMARK]]
local NOTASET = L["No Set"]

local string_format = string.format
local string_find = string.find
local string_sub = string.sub

local LibQTip = LibStub('LibQTip-1.0')
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
 
local dataobj = ldb:NewDataObject(ADDON, {
    type = "data source",
    icon = "Interface\\Addons\\"..ADDON.."\\icon.tga",
    text = "-"
})

local function Button_OnClick(row,arg,button)

	if InCombatLockdown() then return end

	if button == "RightButton" then
		ToggleCharacter('PaperDollFrame')
		PaperDollFrame_SetSidebar(nil, 3)

	elseif button == "LeftButton" then
		C_EquipmentSet.UseEquipmentSet(arg.setID)

	elseif button == "MiddleButton" then
		C_EquipmentSet.SaveEquipmentSet(arg.setID)
		print ("|cffffd200"..ADDON.."|r: " .. L["Set saved"] .. " " .. C_EquipmentSet.GetEquipmentSetInfo(arg.setID))

	end
end
  
local function OnRelease(self)
	LibQTip:Release(self.tooltip)
	self.tooltip = nil
end  
 
local function OnLeave(self)
	if self.tooltip and not self.tooltip:IsMouseOver() then
		self.tooltip:Release()
	end
end  
 
local function anchor_OnEnter(self)
	
	if self.tooltip then
		LibQTip:Release(self.tooltip)
		self.tooltip = nil  
	end
	local row,col
	local tooltip = LibQTip:Acquire(ADDON.."tip", 4, "LEFT", "LEFT", "LEFT","LEFT")

	self.tooltip = tooltip 
	tooltip:SmartAnchorTo(self)
	tooltip:EnableMouse(true)
	tooltip.OnRelease = OnRelease
	tooltip.OnLeave = OnLeave
	tooltip:SetAutoHideDelay(.1, self)

	local numEquipmentSets = C_EquipmentSet.GetNumEquipmentSets()
	if numEquipmentSets > 0 then
	
		arg = {}
		
		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,L["Equipment Sets:"],"CENTER",4)
		tooltip:SetCellTextColor(row,1,1,1,0)			
		
		row,col = tooltip:AddLine("")		
		row,col = tooltip:AddLine("")

		row,col = tooltip:AddLine("name","nr : eq","id","*")
		tooltip:SetLineTextColor(row, 0, 1, 0)
		row,col = tooltip:AddLine("")
		
		for i, setID in next, C_EquipmentSet.GetEquipmentSetIDs() do

			local name, icon, _, isEquipped, numItems, numEquipped, _, _, _ = C_EquipmentSet.GetEquipmentSetInfo(setID)
			
			if isEquipped == true then 
				isEquipped = "*" 
			else 
				isEquipped = ""
			end
			
			row,col = tooltip:AddLine(string_format("|T%s:0|t",icon) .. " " .. name,numItems.." : "..numEquipped,setID,isEquipped)

			arg[row] = { setID = setID, name = name }
			tooltip:SetLineScript(row, 'OnMouseUp', Button_OnClick, arg[row])
  
		end
		
		row,col = tooltip:AddLine("")
		row,col = tooltip:AddLine("")
		
		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,"|cFF00FF00"..L["Left-Click"].."|r","LEFT",1)
		tooltip:SetCell(row,2,"|cFFFFFF00"..L["Equip Set"].."|r","RIGHT",3)
		
		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,"|cFF00FF00"..L["Right-Click"].."|r","LEFT",1)
		tooltip:SetCell(row,2,"|cFFFFFF00"..L["Set Manager"].."|r","RIGHT",3)
		
		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,"|cFF00FF00"..L["Middle-Click"].."|r","LEFT",1)
		tooltip:SetCell(row,2,"|cFFFFFF00"..L["Save Set"].."|r","RIGHT",3)
		

	else 

		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,L["EMPTY"],"CENTER",4)
		tooltip:SetCellTextColor(row,1,1,1,0)		
	
	end
	
	row,col = tooltip:Show()
	
end

dataobj.OnClick = function(self, button)

	if InCombatLockdown() then return end

	if button == "RightButton" then
		ToggleCharacter('PaperDollFrame')
		PaperDollFrame_SetSidebar(nil, 3)
	end
end

dataobj.OnEnter = function(self)
    anchor_OnEnter(self)
end
 
dataobj.OnLeave = function(self)
    -- Null operation: Some LDB displays get cranky if this method is missing.
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
frame:RegisterEvent('UNIT_INVENTORY_CHANGED')
frame:RegisterEvent('EQUIPMENT_SETS_CHANGED')
frame:SetScript("OnEvent", function(self, event, ...)
	for index, setID in next, C_EquipmentSet.GetEquipmentSetIDs() do
		local name, icon, _, isEquipped, _, _, _, _, _ = C_EquipmentSet.GetEquipmentSetInfo(setID)		

		if isEquipped == true then 		
			dataobj.text = name
			dataobj.icon = icon
			return
		end
	end	
	dataobj.text = NOTASET
	dataobj.icon = QUESTION_MARK_ICON
end)